@@ -3,6 +3,10 @@
 ## Copyright (C) Philippe Biondi <phil@secdev.org>
 ## This program is published under a GPLv2 license
 
+"""
+Packet holding data in Abstract Syntax Notation (ASN.1).
+"""
+
 from packet import *
 
 class ASN1_Packet(Packet):
@@ -12,7 +16,9 @@ class ASN1_Packet(Packet):
         flist = self.ASN1_root.get_fields_list()
         self.do_init_fields(flist)
         self.fields_desc = flist    
-    def do_build(self):
+    def self_build(self):
+        if self.raw_packet_cache is not None:
+            return self.raw_packet_cache
         return self.ASN1_root.build(self)    
     def do_dissect(self, x):
         return self.ASN1_root.dissect(self, x)
