@@ -3,6 +3,10 @@
 ## Copyright (C) Philippe Biondi <phil@secdev.org>
 ## This program is published under a GPLv2 license
 
+"""
+Classes and functions for layer 2 protocols.
+"""
+
 import os,struct,time
 from scapy.base_classes import Net
 from scapy.config import conf
@@ -46,6 +50,7 @@ def getmacbyip(ip, chainCC=0):
     """Return MAC address corresponding to a given IP address"""
     if isinstance(ip,Net):
         ip = iter(ip).next()
+    ip = inet_ntoa(inet_aton(ip))
     tmp = map(ord, inet_aton(ip))
     if (tmp[0] & 0xf0) == 0xe0: # mcast @
         return "01:00:5e:%.2x:%.2x:%.2x" % (tmp[1]&0x7f,tmp[2],tmp[3])
@@ -132,7 +137,7 @@ class Ether(Packet):
     name = "Ethernet"
     fields_desc = [ DestMACField("dst"),
                     SourceMACField("src"),
-                    XShortEnumField("type", 0x0000, ETHER_TYPES) ]
+                    XShortEnumField("type", 0x9000, ETHER_TYPES) ]
     def hashret(self):
         return struct.pack("H",self.type)+self.payload.hashret()
     def answers(self, other):
@@ -219,7 +224,7 @@ class Dot1Q(Packet):
     def default_payload_class(self, pay):
         if self.type <= 1500:
             return LLC
-        return Raw
+        return conf.raw_layer
     def extract_padding(self,s):
         if self.type <= 1500:
             return s[:self.type],s[self.type:]
@@ -348,27 +353,44 @@ class ARP(Packet):
                  
 conf.neighbor.register_l3(Ether, ARP, lambda l2,l3: getmacbyip(l3.pdst))
 
+class GRErouting(Packet):
+    name = "GRE routing informations"
+    fields_desc = [ ShortField("address_family",0),
+                    ByteField("SRE_offset", 0),
+                    FieldLenField("SRE_len", None, "routing_info", "B"),
+                    StrLenField("routing_info", "", "SRE_len"),
+                    ]
+
+
 class GRE(Packet):
     name = "GRE"
-    fields_desc = [ BitField("chksumpresent",0,1),
-                    BitField("reserved0",0,12),
+    fields_desc = [ BitField("chksum_present",0,1),
+                    BitField("routing_present",0,1),
+                    BitField("key_present",0,1),
+                    BitField("seqnum_present",0,1),
+                    BitField("strict_route_source",0,1),
+                    BitField("recursion_control",0,3),
+                    BitField("flags",0,5),
                     BitField("version",0,3),
                     XShortEnumField("proto", 0x0000, ETHER_TYPES),
-                    ConditionalField(XShortField("chksum",None),lambda pkt:pkt.chksumpresent==1),
-                    ConditionalField(XShortField("reserved1",None),lambda pkt:pkt.chksumpresent==1),
+                    ConditionalField(XShortField("chksum",None), lambda pkt:pkt.chksum_present==1 or pkt.routing_present==1),
+                    ConditionalField(XShortField("offset",None), lambda pkt:pkt.chksum_present==1 or pkt.routing_present==1),
+                    ConditionalField(XIntField("key",None), lambda pkt:pkt.key_present==1),
+                    ConditionalField(XIntField("seqence_number",None), lambda pkt:pkt.seqnum_present==1),
                     ]
     def post_build(self, p, pay):
         p += pay
-        if self.chksumpresent and self.chksum is None:
+        if self.chksum_present and self.chksum is None:
             c = checksum(p)
             p = p[:4]+chr((c>>8)&0xff)+chr(c&0xff)+p[6:]
         return p
-            
+
 
 
 
 bind_layers( Dot3,          LLC,           )
 bind_layers( Ether,         LLC,           type=122)
+bind_layers( Ether,         LLC,           type=34928)
 bind_layers( Ether,         Dot1Q,         type=33024)
 bind_layers( Ether,         Ether,         type=1)
 bind_layers( Ether,         ARP,           type=2054)
@@ -384,6 +406,9 @@ bind_layers( GRE,           Dot1Q,         proto=33024)
 bind_layers( GRE,           Ether,         proto=1)
 bind_layers( GRE,           ARP,           proto=2054)
 bind_layers( GRE,           EAPOL,         proto=34958)
+bind_layers( GRE,           GRErouting,    { "routing_present" : 1 } )
+bind_layers( GRErouting,    conf.raw_layer,{ "address_family" : 0, "SRE_len" : 0 })
+bind_layers( GRErouting,    GRErouting,    { } )
 bind_layers( EAPOL,         EAP,           type=0)
 bind_layers( LLC,           STP,           dsap=66, ssap=66, ctrl=3)
 bind_layers( LLC,           SNAP,          dsap=170, ssap=170, ctrl=3)
@@ -397,8 +422,8 @@ conf.l2types.register(ARPHDR_ETHER, Ether)
 conf.l2types.register_num2layer(ARPHDR_METRICOM, Ether)
 conf.l2types.register_num2layer(ARPHDR_LOOPBACK, Ether)
 conf.l2types.register_layer2num(ARPHDR_ETHER, Dot3)
-conf.l2types.register(113, CookedLinux)
 conf.l2types.register(144, CookedLinux)  # called LINUX_IRDA, similar to CookedLinux
+conf.l2types.register(113, CookedLinux)
 
 conf.l3types.register(ETH_P_ARP, ARP)
 
@@ -449,7 +474,7 @@ Set cache=True if you want arping to modify internal ARP-Cache"""
 
     if cache and ans is not None:
         for pair in ans:
-            arp_cache[pair[1].psrc] = (pair[1].hwsrc, time.time())
+            conf.netcache.arp_cache[pair[1].psrc] = (pair[1].hwsrc, time.time())
     if verbose:
         ans.show()
     return ans,unans
@@ -475,13 +500,30 @@ def promiscping(net, timeout=2, fake_bcast="ff:ff:ff:ff:ff:fe", **kargs):
 
 
 class ARP_am(AnsweringMachine):
+    """Fake ARP Relay Daemon (farpd)
+
+    example:
+    To respond to an ARP request for 192.168.100 replying on the
+    ingress interface;
+      farpd(IP_addr='192.168.1.100',ARP_addr='00:01:02:03:04:05')
+    To respond on a different interface add the interface parameter
+      farpd(IP_addr='192.168.1.100',ARP_addr='00:01:02:03:04:05',iface='eth0')
+    To respond on ANY arp request on an interface with mac address ARP_addr
+      farpd(ARP_addr='00:01:02:03:04:05',iface='eth1')
+    To respond on ANY arp request with my mac addr on the given interface
+      farpd(iface='eth1')
+
+    Optional Args
+     inter=<n>   Interval in seconds between ARP replies being sent
+    
+    """
+
     function_name="farpd"
     filter = "arp"
     send_function = staticmethod(sendp)
 
-    def parse_options(self, IP_addr=None, iface=None, ARP_addr=None):
+    def parse_options(self, IP_addr=None, ARP_addr=None):
         self.IP_addr=IP_addr
-        self.iface=iface
         self.ARP_addr=ARP_addr
 
     def is_request(self, req):
@@ -492,26 +534,43 @@ class ARP_am(AnsweringMachine):
     def make_reply(self, req):
         ether = req.getlayer(Ether)
         arp = req.getlayer(ARP)
-        iff,a,gw = conf.route.route(arp.psrc)
-        if self.iface != None:
-            iff = iface
-        ARP_addr = self.ARP_addr
-        IP_addr = arp.pdst
+
+        if self.optsend.has_key('iface'):
+            iff = self.optsend.get('iface')
+        else:
+            iff,a,gw = conf.route.route(arp.psrc)
+        self.iff = iff
+        if self.ARP_addr is None:
+            try:
+                ARP_addr = get_if_hwaddr(iff)
+            except:
+                ARP_addr = "00:00:00:00:00:00"
+                pass
+        else:
+            ARP_addr = self.ARP_addr
         resp = Ether(dst=ether.src,
                      src=ARP_addr)/ARP(op="is-at",
                                        hwsrc=ARP_addr,
-                                       psrc=IP_addr,
+                                       psrc=arp.pdst,
                                        hwdst=arp.hwsrc,
-                                       pdst=arp.pdst)
+                                       pdst=arp.psrc)
         return resp
 
-    def sniff(self):
-        sniff(iface=self.iface, **self.optsniff)
+    def send_reply(self, reply):
+        if self.optsend.has_key('iface'):
+            self.send_function(reply, **self.optsend)
+        else:
+            self.send_function(reply, iface=self.iff, **self.optsend)
+
+    def print_reply(self, req, reply):
+        print "%s ==> %s on %s" % (req.summary(),reply.summary(),self.iff)
+
 
 @conf.commands.register
 def etherleak(target, **kargs):
     """Exploit Etherleak flaw"""
-    return srpflood(Ether()/ARP(pdst=target), prn=lambda (s,r): Padding in r and hexstr(r[Padding].load),
+    return srpflood(Ether()/ARP(pdst=target), 
+                    prn=lambda (s,r): conf.padding_layer in r and hexstr(r[conf.padding_layer].load),
                     filter="arp", **kargs)
 
 
