@@ -3,6 +3,9 @@
 ## Copyright (C) Philippe Biondi <phil@secdev.org>
 ## This program is published under a GPLv2 license
 
+"""
+Unit testing infrastructure for Scapy
+"""
 
 import sys,getopt,imp
 import bz2, base64, os.path, time, traceback, zlib, sha
@@ -98,6 +101,7 @@ class Format(EnumClass):
     ANSI  = 2
     HTML  = 3
     LATEX = 4
+    XUNIT = 5
 
 
 #### TEST CLASSES ####
@@ -315,6 +319,7 @@ def run_campaign(test_campaign, get_interactive_session, verb=2):
     if verb:
         print >>sys.stderr,"Campaign CRC=%(crc)s  SHA=%(sha)s" % test_campaign
         print >>sys.stderr,"PASSED=%i FAILED=%i" % (passed, failed)
+    return failed
 
 
 #### INFO LINES ####
@@ -342,10 +347,11 @@ def campaign_to_TEXT(test_campaign):
     output += "Passed=%(passed)i\nFailed=%(failed)i\n\n%(headcomments)s\n" % test_campaign
     
     for testset in test_campaign:
-        output += "######\n## %(name)s\n######\n%(comments)s\n\n" % testset
-        for t in testset:
-            if t.expand:
-                output += "###(%(num)03i)=[%(result)s] %(name)s\n%(comments)s\n%(output)s\n\n" % t
+        if any(t.expand for t in testset):
+            output += "######\n## %(name)s\n######\n%(comments)s\n\n" % testset
+            for t in testset:
+                if t.expand:
+                    output += "###(%(num)03i)=[%(result)s] %(name)s\n%(comments)s\n%(output)s\n\n" % t
 
     return output
  
@@ -355,13 +361,27 @@ def campaign_to_ANSI(test_campaign):
     output += "Passed=%(passed)i\nFailed=%(failed)i\n\n%(headcomments)s\n" % test_campaign
     
     for testset in test_campaign:
-        output += "######\n## %(name)s\n######\n%(comments)s\n\n" % testset
-        for t in testset:
-            if t.expand:
-                output += "###(%(num)03i)=[%(result)s] %(name)s\n%(comments)s\n%(output)s\n\n" % t
+        if any(t.expand for t in testset):
+            output += "######\n## %(name)s\n######\n%(comments)s\n\n" % testset
+            for t in testset:
+                if t.expand:
+                    output += "###(%(num)03i)=[%(result)s] %(name)s\n%(comments)s\n%(output)s\n\n" % t
 
     return output
 
+def campaign_to_xUNIT(test_campaign):
+    output='<?xml version="1.0" encoding="UTF-8" ?>\n<testsuite>\n'
+    for testset in test_campaign:
+        for t in testset:
+            output += ' <testcase classname="%s"\n' % testset.name.encode("string_escape").replace('"',' ')
+            output += '           name="%s"\n' % t.name.encode("string_escape").replace('"',' ')
+            output += '           duration="0">\n' % t
+            if not t.res:
+                output += '<error><![CDATA[%(output)s]]></error>\n' % t
+            output += "</testcase>\n"
+    output += '</testsuite>'
+    return output
+
 
 def campaign_to_HTML(test_campaign, local=0):
     output = """<html>
@@ -585,6 +605,7 @@ def main(argv):
         Format.ANSI: scapy.autorun_get_ansi_interactive_session,
         Format.HTML: scapy.autorun_get_html_interactive_session,
         Format.LATEX: scapy.autorun_get_latex_interactive_session,
+        Format.XUNIT: scapy.autorun_get_text_interactive_session,
         }
 
     # Parse test file
@@ -617,7 +638,7 @@ def main(argv):
 
     # Run tests
     test_campaign.output_file = OUTPUTFILE
-    run_campaign(test_campaign, autorun_func[FORMAT], verb=VERB)
+    result = run_campaign(test_campaign, autorun_func[FORMAT], verb=VERB)
 
     # Shrink passed
     if ONLYFAILED:
@@ -636,9 +657,12 @@ def main(argv):
         output = campaign_to_HTML(test_campaign, local=LOCAL)
     elif FORMAT == Format.LATEX:
         output = campaign_to_LATEX(test_campaign)
+    elif FORMAT == Format.XUNIT:
+        output = campaign_to_xUNIT(test_campaign)
 
     OUTPUTFILE.write(output)
     OUTPUTFILE.close()
+    return result
 
 if __name__ == "__main__":
-    main(sys.argv[1:])
+    exit(main(sys.argv[1:]))
